using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Data.SQLite;
using System.IO;

using System.Diagnostics;

using System.Xml;

using System.Globalization;

namespace GMDL
{
    public partial class iPhoneF : Form
    {
        public iPhoneF(string map_name, string out_name_p)
        {
            InitializeComponent();

            map_1.Text = map_name;
            out_name.Text = out_name_p;
        }

        private bool ThumbnailCallback()
        {
            return false;
        }

        public bool ExtractGPSFSiphone(string map_dir, int layer, string force_ext, bool set_pos)
        {
            bool extracted_ok = true;

            if (ImgDump.Checked)
                new DirectoryInfo(out_name.Text + "_" + layer.ToString()).Create();

            DirectoryInfo td = new DirectoryInfo(map_dir);
            if (td.Exists == false)
                extracted_ok = false; // no directory
            else
                if (new FileInfo(map_dir + "/GPSFS").Exists == false)
                    extracted_ok = false; // gpsfs does not exist

            if (extracted_ok)
            {
                BinaryReader sr = new BinaryReader(new FileStream(map_dir + "/GPSFS", FileMode.Open));
                try
                {
                    string gpsfsv = "";
                    Int64 x = -1;
                    Int64 y = -1;
                    Int64 basezoom = 0;
                    Int64 filetype = -1;
                    Int64 maxsize = -1;

                    gpsfsv += sr.ReadChar();
                    gpsfsv += sr.ReadChar();
                    gpsfsv += sr.ReadChar();
                    gpsfsv += sr.ReadChar();
                    gpsfsv += sr.ReadChar();
                    gpsfsv += sr.ReadChar();
                    gpsfsv += sr.ReadChar();
                    gpsfsv += sr.ReadChar();

                    x = sr.ReadInt64();
                    y = sr.ReadInt64();
                    basezoom = sr.ReadInt64();
                    filetype = sr.ReadInt64();
                    maxsize = sr.ReadInt64();

                    int top_dir = 0;
                    int tiles = 0;
                    while (tiles < maxsize)
                    {
                        top_dir++;
                        tiles = System.Convert.ToInt32(TotalTiles(top_dir));
                    }

                    int x_off = 0;
                    int y_off = 0;

                    if (x >= 0 && y >= 0)
                    {
                        x_off = System.Convert.ToInt32(x);
                        y_off = System.Convert.ToInt32(y);
                    } // else 00 is ok

                    int zoom_dir = 0;
                    while (Math.Pow(2, zoom_dir) != top_dir)
                    {
                        zoom_dir++;
                    }
                    zoom_dir++;

                    if (x >= 0 && y >= 0)
                    {
                        zoom_dir = 17 - System.Convert.ToInt32(basezoom);
                    } // else 00 is ok

                    string ext;
                    if (filetype == 0)
                        ext = "png";
                    else
                        ext = "jpg";

                    if (force_ext != "")
                        ext = force_ext;

                    if (set_pos && x >= 0 && y >= 0)// && lat_c.Text == "" && lon_c.Text == "" && zoom_c.Text == "" && x >= 0 && y >= 0)
                    {
                        int top_zoom = System.Convert.ToInt32(basezoom);
                        for (int i = 1; i <= top_dir; i *= 2)
                        {
                            top_zoom++;
                        }
                        int tile_size = System.Convert.ToInt32(Math.Pow(2, top_zoom - basezoom));

                        decimal[] Tcoord = LLXY.getLatLong(System.Convert.ToInt32(x), System.Convert.ToInt32(y), System.Convert.ToInt32(basezoom)); // top left
                        decimal[] Bcoord = LLXY.getLatLong(System.Convert.ToInt32(x) + tile_size, System.Convert.ToInt32(y) + tile_size, System.Convert.ToInt32(basezoom)); // bottom right
                        
                        decimal lat = ((decimal)Tcoord[1] + (decimal)Bcoord[1]) / 2;
                        decimal lon = ((decimal)Tcoord[0] + (decimal)Bcoord[0]) / 2;

                        if (lat < (decimal)0.0000001 && lat > (decimal)-0.0000001)
                            lat = 0;
                        if (lon < (decimal)0.0000001 && lon > (decimal)-0.0000001)
                            lon = 0;

                        lat_c.Text = lat.ToString().Replace(",", ".");
                        lon_c.Text = lon.ToString().Replace(",", ".");
                        zoom_c.Text = (19 - top_zoom).ToString();

                        Application.DoEvents();
                    }

                    // get N + 1 coords
                    DataTable IDX = new DataTable();
                    IDX.Columns.Add("x", typeof(Int32));
                    IDX.Columns.Add("y", typeof(Int32));
                    IDX.Columns.Add("z", typeof(Int32));
                    IDX.Columns.Add("IDXfile", typeof(Int32));
                    IDX.Columns.Add("IDXoffset", typeof(Int32));

                    // table of tiles loc, with length, length++ (in order)
                    int zoom = 1;
                    while (zoom <= top_dir)
                    {
                        int row = 0;
                        while (row < (top_dir * 2) / zoom)
                        {
                            int col = 0;
                            while (col < (top_dir * 2) / zoom)
                            {
                                DataRow new_row = IDX.NewRow();
                                new_row["x"] = x_off + col;
                                new_row["y"] = y_off + row;
                                new_row["z"] = zoom_dir;
                                new_row["IDXfile"] = sr.ReadInt32();
                                new_row["IDXoffset"] = sr.ReadInt32();
                                IDX.Rows.Add(new_row);

                                col++;
                            }
                            row++;
                        }
                        zoom_dir--;
                        x_off /= 2;
                        y_off /= 2;
                        zoom = zoom * 2; // will round down to 0
                    }

                    DataRow new_rowE = IDX.NewRow();
                    new_rowE["IDXfile"] = sr.ReadInt32(); // the last one
                    new_rowE["IDXoffset"] = sr.ReadInt32(); // the last one
                    IDX.Rows.Add(new_rowE);

                    int IDXfile = 0;

                    string dest = out_name.Text + "/MapTiles.sqlitedb";
                    SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
                    sql.Open();

                    if (1 == 2)
                    {
                        SQLiteCommand cmd0 = sql.CreateCommand();
                        cmd0.CommandText = "SELECT zoom, x, y, flags, length FROM images";

                        using (SQLiteDataReader reader = cmd0.ExecuteReader())
                        {
                            while (reader.Read())
                            {
                                Console.WriteLine(reader[0].ToString() + "###" + reader[1].ToString() + "###" + reader[2].ToString() + "###" + reader[3].ToString() + "###" + reader[4].ToString() + "###");
                            }
                        }
                    }

                    if (1 == 2)
                    {
                        SQLiteCommand cmd0b = sql.CreateCommand();
                        cmd0b.CommandText = "SELECT * FROM version";

                        using (SQLiteDataReader reader = cmd0b.ExecuteReader())
                        {
                            while (reader.Read())
                            {
                                Console.WriteLine(reader[0].ToString() + "###");
                            }
                        }
                    }

                    // uses blank source
                    //SQLiteCommand cmdD1 = sql.CreateCommand();
                    //cmdD1.CommandText = "DELETE FROM images";
                    //cmdD1.ExecuteNonQuery();

                    // need version
                    //SQLiteCommand cmdD2 = sql.CreateCommand();
                    //cmdD2.CommandText = "DELETE FROM version";
                    //cmdD2.ExecuteNonQuery();

                    string flags = layer.ToString();
                    System.Drawing.Imaging.ImageFormat imgf;
                    if (ext == "jpg")
                        imgf = System.Drawing.Imaging.ImageFormat.Jpeg;
                    else
                        imgf = System.Drawing.Imaging.ImageFormat.Png;

                    SQLiteTransaction trans = sql.BeginTransaction();
                    trans.Commit();
                    if (CommitOnce.Checked)
                        trans = sql.BeginTransaction();

                    for (int i = 0; i < maxsize; i++)
                    {
                        Process.Text = "On " + i.ToString() + " of " + maxsize.ToString() + " - " + (100 * i / maxsize).ToString("0") + "%";
                        Application.DoEvents();
                        if ((int)IDX.Rows[i]["IDXoffset"] >= 0)
                        {
                            int length;
                            if ((int)IDX.Rows[i + 1]["IDXoffset"] > 0)
                            {
                                length = (int)IDX.Rows[i + 1]["IDXoffset"] - (int)IDX.Rows[i]["IDXoffset"];
                            }
                            else if ((int)IDX.Rows[i + 1]["IDXoffset"] < 0)
                            {
                                length = (int)IDX.Rows[i + 1]["IDXoffset"] * -1;
                            }
                            else // if ((int)IDX.Rows[i + 1]["IDXoffset"] == 0)
                            {
                                length = System.Convert.ToInt32(sr.BaseStream.Length) - (int)IDX.Rows[i]["IDXoffset"];
                            }

                            if (IDXfile != (int)IDX.Rows[i]["IDXfile"])
                            {
                                IDXfile = (int)IDX.Rows[i]["IDXfile"];
                                sr.Close();
                                sr = new BinaryReader(new FileStream(map_dir + "/GPSFS" + IDXfile.ToString(), FileMode.Open));
                            }

                            Bitmap image = new Bitmap(new MemoryStream(sr.ReadBytes(System.Convert.ToInt32(length))));
                            if (1 == 2)
                                image = (Bitmap)image.GetThumbnailImage(image.Width, image.Height, new Image.GetThumbnailImageAbort(ThumbnailCallback), System.IntPtr.Zero);

                            int z = (int)IDX.Rows[i]["z"] + 1;
                            int xS = (int)IDX.Rows[i]["x"] * 4;
                            int yS = (int)IDX.Rows[i]["y"] * 4;

                            if (layer == 2 && No1xMap.Checked && z >= 18)
                            {
                                // do nothing - this could well be faster
                            }
                            else
                            {
                                if (CommitOnTile.Checked)
                                    trans = sql.BeginTransaction();

                                SubmitDB(0, 0, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(0, 1, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(0, 2, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(0, 3, xS, yS, z, flags, image, imgf, sql, trans);

                                SubmitDB(1, 0, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(1, 1, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(1, 2, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(1, 3, xS, yS, z, flags, image, imgf, sql, trans);

                                SubmitDB(2, 0, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(2, 1, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(2, 2, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(2, 3, xS, yS, z, flags, image, imgf, sql, trans);

                                SubmitDB(3, 0, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(3, 1, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(3, 2, xS, yS, z, flags, image, imgf, sql, trans);
                                SubmitDB(3, 3, xS, yS, z, flags, image, imgf, sql, trans);

                                if (CommitOnTile.Checked)
                                    trans.Commit();
                            }

                            image.Dispose();
                        }
                    }
                    if (CommitOnce.Checked)
                    {
                        Process.Text = "Committing . . . wait";
                        Application.DoEvents();
                        trans.Commit();
                    }
                    Process.Text = "Process (click here, and wait)";

                    sql.Close();
                }
                catch
                {
                    extracted_ok = false;
                }

                sr.Close();
            }

            return extracted_ok;
        }

        private void SubmitDB(int x_off, int y_off, int x, int y, int z, string flags, Bitmap image, System.Drawing.Imaging.ImageFormat imgf, SQLiteConnection sql, SQLiteTransaction trans)
        {
            Rectangle r00 = new Rectangle(x_off * 64, y_off * 64, 64, 64);
            Bitmap c00 = image.Clone(r00, image.PixelFormat);
            MemoryStream ms00 = new MemoryStream();
            c00.Save(ms00, imgf);
            /*
            //Info[1] is jpeg; [4] is png
            System.Drawing.Imaging.EncoderParameters Params;// = new System.Drawing.Imaging.EncoderParameters(0);
            System.Drawing.Imaging.ImageCodecInfo[] Info = System.Drawing.Imaging.ImageCodecInfo.GetImageEncoders();
            
            Params = new System.Drawing.Imaging.EncoderParameters(1);
            Params.Param[0] = new System.Drawing.Imaging.EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 5L);

            c00.Save(ms00, Info[1], Params);
            */
            //if (1 == 2 && z != "5")
            //    Console.WriteLine(z + " " + xS + " " + yS + " " + len + " " + i.ToString());
            //if (ms00.Length > 1000)
            //{
                if (ImgDump.Checked)
                {
                    string img_path = out_name.Text + "_" + flags + "\\" + z.ToString() + "_" + (x + x_off).ToString() + "_" + (y + y_off).ToString();
                    c00.Save(img_path, imgf);
                    //c00.Save(img_path, Info[1], Params);
                }

                SQLiteCommand cmd2 = sql.CreateCommand();
                cmd2.CommandText = "INSERT INTO images (zoom, x, y, flags, length, data) VALUES(" + z.ToString() + ", " + (x + x_off).ToString() + ", " + (y + y_off).ToString() + ", " + flags + ", " + ms00.Length.ToString() + ", @img)";
                SQLiteParameter para = new SQLiteParameter("@img");
                cmd2.Parameters.Add(para);
                para.Value = ms00.ToArray();
                cmd2.Transaction = trans;
                cmd2.ExecuteNonQuery();
            //}
            c00.Dispose();
        }

        public Int64 TotalTiles(int top_dir)
        {
            Int64 maxsize = 0;

            int zoom = 1;
            int span = 2;

            while (zoom <= top_dir)
            {
                maxsize += span * span;
                zoom *= 2;
                span *= 2;
            }

            return maxsize;
        }

        LatLonXYcalcs LLXY = new LatLonXYcalcs();

        private void Process_Click(object sender, EventArgs e)
        {
            DirectoryInfo dd = new DirectoryInfo(out_name.Text);
            if (dd.Exists)
            {
                MessageBox.Show("Output directory is already there");
            }
            else
            {
                DateTime start = DateTime.Now;

                dd.Create();
                bool new_db = newDB.Checked;
                if (new_db)
                {
                    string dest = out_name.Text + "/MapTiles.sqlitedb";
                    SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
                    sql.Open();

                    SQLiteCommand cmdD1 = sql.CreateCommand();
                    cmdD1.CommandText = "CREATE TABLE images(zoom int, x int, y int, flags int, length int, data blob)";
                    cmdD1.ExecuteNonQuery();

                    sql.Close();
                }
                else
                {
                    new FileInfo("iPhone/MapTiles.sqlitedb").CopyTo(out_name.Text + "/MapTiles.sqlitedb");
                }
                if (CopyLmSh.Checked)
                    new FileInfo("iPhone/lm.sh").CopyTo(out_name.Text + "/lm.sh");

                string did = "";
                string failed = "";

                string force_ext_map = "";
                if (map_1_png.Checked)
                    force_ext_map = "png";
                if (map_1_jpg.Checked)
                    force_ext_map = "jpg";

                string force_ext_sat = "";
                if (map_2_png.Checked)
                    force_ext_sat = "png";
                if (map_2_jpg.Checked)
                    force_ext_sat = "jpg";

                string force_ext_hyb = "";
                if (map_3_png.Checked)
                    force_ext_hyb = "png";
                if (map_3_jpg.Checked)
                    force_ext_hyb = "jpg";

                if (map_1.Text != "")
                {
                    if (ExtractGPSFSiphone(map_1.Text, 2, force_ext_map, launch_map.Checked))
                        did = "Map ";
                    else
                        failed = "Map ";
                }

                if (map_2.Text != "")
                {
                    if (ExtractGPSFSiphone(map_2.Text, 3, force_ext_sat, launch_sat.Checked))
                        did = did + "Sat";
                    else
                        failed = failed + "Sat";
                }

                if (map_3.Text != "")
                {
                    if (ExtractGPSFSiphone(map_3.Text, 6, force_ext_hyb, launch_hyb.Checked))
                        did = did + "Hyb";
                    else
                        failed = failed + "Hyb";
                }

                // create com.apple.Maps.plist
                string launch_layer = "0";
                if (launch_sat.Checked)
                    launch_layer = "1";
                if (launch_hyb.Checked)
                    launch_layer = "2";

                StreamWriter sw = new StreamWriter(out_name.Text + "/com.apple.Maps.plist");
                sw.WriteLine("<plist>");
                sw.WriteLine("<dict>");
                sw.WriteLine("	<key>LastViewMode</key>");
                sw.WriteLine("	<integer>" + launch_layer + "</integer>");
                if (lat_c.Text == "" || lon_c.Text == "" || zoom_c.Text == "")
                {
                    lat_c.Text = "0.0";
                    lon_c.Text = "0.0";
                    zoom_c.Text = "2";
                }

                sw.WriteLine("	<key>LastViewedLatitude</key>");
                sw.WriteLine("	<real>" + lat_c.Text + "</real>");
                sw.WriteLine("	<key>LastViewedLongitude</key>");
                sw.WriteLine("	<real>" + lon_c.Text + "</real>");
                sw.WriteLine("	<key>LastViewedZoomScale</key>");
                sw.WriteLine("	<real>" + zoom_c.Text + "</real>");

                int placemarks = 0;

                if (XMLplacemarks.Text != "")
                {
                    sw.WriteLine("	<key>SearchResults</key>");
                    sw.WriteLine("	<array>");

                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.Load(XMLplacemarks.Text);

                    foreach (XmlNode res in xmlDoc.GetElementsByTagName("Placemark"))
                    {
                        string name = "";
                        decimal lat = 0;
                        decimal lon = 0;
                        foreach (XmlNode res2 in res.ChildNodes)
                        {
                            if (res2.Name.ToLower() == "name")
                            {
                                name = res2.InnerText;
                            }
                            if (res2.Name.ToLower() == "point")
                            {
                                foreach (XmlNode res3 in res2.ChildNodes)
                                {
                                    if (res3.Name.ToLower() == "coordinates")
                                    {
                                        string lonS = res2.InnerText;
                                        lonS = lonS.Substring(0, lonS.IndexOf(","));
                                        lon = Decimal.Parse(lonS, NumberFormatInfo.InvariantInfo);
                                        if (KMLsky.Checked)
                                            lon = lon * -1;
                                        string latS = res2.InnerText + ",";
                                        latS = latS.Substring(latS.IndexOf(",") + 1, latS.IndexOf(",", latS.IndexOf(",") + 1) - 1 - latS.IndexOf(","));
                                        lat = Decimal.Parse(latS, NumberFormatInfo.InvariantInfo);
                                    }
                                }
                            }
                        }
                        sw.WriteLine("		<dict>");
                        sw.WriteLine("			<key>LatitudeE6</key>");
                        sw.WriteLine("			<integer>" + System.Convert.ToInt32(lat * (decimal)1000000).ToString() + "</integer>");
                        sw.WriteLine("			<key>LongitudeE6</key>");
                        sw.WriteLine("			<integer>" + System.Convert.ToInt32(lon * (decimal)1000000).ToString() + "</integer>");
                        sw.WriteLine("			<key>Name</key>");
                        sw.WriteLine("			<string>" + name + "</string>");
                        sw.WriteLine("		</dict>");
                        placemarks++;
                    }
                    sw.WriteLine("	</array>");
                }

                sw.WriteLine("</dict>");
                sw.WriteLine("</plist>");
                sw.Close();

                if (new_db)
                {
                    string dest = out_name.Text + "/MapTiles.sqlitedb";
                    SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
                    sql.Open();

                    Process.Text = "Building index . . . wait";
                    Application.DoEvents();
                    SQLiteCommand cmdD2 = sql.CreateCommand();
                    cmdD2.CommandText = "CREATE INDEX index1 on images (zoom,x,y,flags)";
                    cmdD2.ExecuteNonQuery();
                    Process.Text = "Process (click here, and wait)";

                    cmdD2.CommandText = "CREATE TABLE version(version int)";
                    cmdD2.ExecuteNonQuery();

                    if (v1.Text != "")
                    {
                        cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v1.Text + ")";
                        cmdD2.ExecuteNonQuery();
                    }
                    if (v2.Text != "")
                    {
                        cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v2.Text + ")";
                        cmdD2.ExecuteNonQuery();
                    }
                    if (v3.Text != "")
                    {
                        cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v3.Text + ")";
                        cmdD2.ExecuteNonQuery();
                    }

                    sql.Close();
                }

                DateTime end = DateTime.Now;

                TimeSpan ts = end - start;

                string msg_pm = "";
                if (placemarks > 0)
                {
                    msg_pm = msg_pm + "\n\n\n" + placemarks.ToString() + " Placemarks (Pins under List)";
                }

                string msg = "Did: " + did;
                if (failed != "")
                    msg = msg + "\n\nFailed: " + failed;
                MessageBox.Show(msg + "\n\n" + ts.ToString() + msg_pm);

                lat_c.Text = "";
                lon_c.Text = "";
                zoom_c.Text = "";
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            string dest = out_name.Text + "/MapTiles.sqlitedb";
            SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
            sql.Open();

            SQLiteCommand cmdD1 = sql.CreateCommand();
            cmdD1.CommandText = "UPDATE images SET flags = 2";
            cmdD1.ExecuteNonQuery();

            sql.Close();

            MessageBox.Show("Done");
        }

        private void PXLlink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process proc = new Process();
            proc.StartInfo.FileName = PXLlink.Text;
            proc.Start();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            string dest = out_name.Text + "/MapTiles.sqlitedb";
            SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
            sql.Open();

            SQLiteCommand cmd0b = sql.CreateCommand();
            cmd0b.CommandText = "SELECT * FROM version";

            string v = "";

            using (SQLiteDataReader reader = cmd0b.ExecuteReader())
            {
                while (reader.Read())
                {
                    v = v + reader[0].ToString() + ",";
                }
            }

            MessageBox.Show(v);

            sql.Close();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            DataTable layers = new DataTable();
            layers.Columns.Add("layer", typeof(String));

            string dest = out_name.Text + "/MapTiles.sqlitedb";
            SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
            sql.Open();

            SQLiteCommand cmd0b = sql.CreateCommand();
            cmd0b.CommandText = "SELECT flags FROM images";

            
            using (SQLiteDataReader reader = cmd0b.ExecuteReader())
            {
                while (reader.Read())
                {
                    if (layers.Select("layer = " + reader[0].ToString()).Length == 0)
                        layers.Rows.Add(new object[] { reader[0].ToString() });
                }
            }

            string v = "";
            foreach (DataRow row in layers.Rows)
            {
                v = v + row["layer"].ToString() + ",";
            }

            MessageBox.Show(v);

            sql.Close();
        }

        private void SetKML_Click(object sender, EventArgs e)
        {
            string dir = System.IO.Directory.GetCurrentDirectory();

            OpenFileDialog file = new OpenFileDialog();
            file.Filter = "KML File - Placemarks (*.kml)|*.kml";
            file.ShowDialog();

            System.IO.Directory.SetCurrentDirectory(dir);

            if (file.FileName != "")
            {
                XMLplacemarks.Text = file.FileName;
            }
        }

        private void SetMapDir_Click(object sender, EventArgs e)
        {
            string dir = System.IO.Directory.GetCurrentDirectory();

            FolderBrowserDialog file = new FolderBrowserDialog();
            file.ShowDialog();

            System.IO.Directory.SetCurrentDirectory(dir);

            if (file.SelectedPath != "")
            {
                out_name.Text = file.SelectedPath;
            }
        }

        private void fix_version_Click(object sender, EventArgs e)
        {
            string dest = out_name.Text + "/MapTiles.sqlitedb";
            SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
            sql.Open();

            SQLiteCommand cmdD2 = sql.CreateCommand();
            cmdD2.CommandText = "DELETE FROM version";
            cmdD2.ExecuteNonQuery();

            if (v1.Text != "")
            {
                cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v1.Text + ")";
                cmdD2.ExecuteNonQuery();
            }
            if (v2.Text != "")
            {
                cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v2.Text + ")";
                cmdD2.ExecuteNonQuery();
            }
            if (v3.Text != "")
            {
                cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v3.Text + ")";
                cmdD2.ExecuteNonQuery();
            }

            sql.Close();

            MessageBox.Show("Done");
        }

        private void fix_version_subdirs_Click(object sender, EventArgs e)
        {
            string done = "";
            string failed = "";
            foreach (DirectoryInfo dest_dir in new DirectoryInfo(out_name.Text).GetDirectories())
            {
                try
                {
                    string dest = dest_dir.FullName + "/MapTiles.sqlitedb";
                    SQLiteConnection sql = new SQLiteConnection("Data Source=" + dest);
                    sql.Open();

                    SQLiteCommand cmdD2 = sql.CreateCommand();
                    cmdD2.CommandText = "DELETE FROM version";
                    cmdD2.ExecuteNonQuery();

                    if (v1.Text != "")
                    {
                        cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v1.Text + ")";
                        cmdD2.ExecuteNonQuery();
                    }
                    if (v2.Text != "")
                    {
                        cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v2.Text + ")";
                        cmdD2.ExecuteNonQuery();
                    }
                    if (v3.Text != "")
                    {
                        cmdD2.CommandText = "INSERT INTO version (version) VALUES(" + v3.Text + ")";
                        cmdD2.ExecuteNonQuery();
                    }

                    sql.Close();
                    done = done + dest_dir.Name + "\n";
                }
                catch
                {
                    failed = failed + dest_dir.Name + "\n";
                }
            }

            MessageBox.Show("Done:\n\n" + done + "\n\nFailed:\n\n" + failed);
        }

    }
}